/*---------------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.
  
  Copyright (c) Jack Botner and licensed under the MIT license; see
  http://opensource.org/licenses/mit-license.php

  calendar.c

  Some calendar routines.
---------------------------------------------------------------------*/
#include <stdint.h>
#include "calendar.h"

// Global variables

static unsigned char daysmonth[12] =
{
  31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};

// Following tables used to compute day of week
static unsigned char months_table[12] =
{
  0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5
};

static unsigned char months_table_ly[12] =
{
  6, 2, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5
};

// Internal functions

static uint8_t dayofweek( uint8_t ucYear, uint8_t ucMonth, uint8_t ucDay );
static uint8_t days_in_month( uint8_t ucMonth, uint8_t ucYear );
static uint8_t isleapyear( uint8_t ucYear );

/*---------------------------------------------------------------------
  days_in_month(...)

  Returns the number of days in the month, given the month and year.

  Input: Year  - 0..99  
		 Month - 1..12  

  Returns: 28..31
---------------------------------------------------------------------*/
static uint8_t days_in_month( uint8_t ucMonth, uint8_t ucYear )
{
  uint8_t ii, ucDaysMon;

  if ( ucMonth < 1 || ucMonth > 12 )          // bad input
    return 0;

  ii = ucMonth - 1;
  ucDaysMon = daysmonth[ii];

  if ( ucMonth == 2 )
  {
    if ( isleapyear( ucYear ) )
      ++ucDaysMon;
  }
  
  return ucDaysMon;
}

/*--------------------------------------------------------------------------
  isdaylighttime()
  
  Time can be either standard or daylight savings. In this locale, 
  EDT begins on the 2nd Sunday in March and ends on the 1st Sunday in 
  November.

  Input: Year  - 0..99  
		     Month - 1..12  
         Day   - 1..31
		     Hour  - 0..23

  Returns: 1 if daylight savings time, 0 if standard time.
--------------------------------------------------------------------------*/
uint8_t isdaylighttime( uint8_t ucYear, uint8_t ucMonth, uint8_t ucDay, 
						uint8_t ucHour )
{
  uint8_t ucDow, ucDy;

  if ( ucMonth < 3 || ucMonth > 11 ) return 0;
  if ( ucMonth > 3 && ucMonth < 11 ) return 1;

  if ( ucMonth == 3 )
  {
    // Its March so find the 2nd Sunday.
    // What day is March 1st?
    ucDy = 1;
    ucDow = dayofweek( ucYear, ucMonth, ucDy );
    // dayofweek returns 0..6, where 0 = Sunday
    if ( ucDow )                        // not Sunday
      ucDy = 8 - ucDow;
    ucDy += 7;                          // 2nd Sunday
    if ( ucDay > ucDy ) return 1;
    if ( ucDay < ucDy ) return 0;
    if ( ucHour > 1 ) return 1;         // after 2:00
    return 0;
  }

  // It's November, find the 1st Sunday.
  ucDy = 1;
  ucDow = dayofweek( ucYear, ucMonth, ucDy );
  // dayofweek returns 0..6, where 0 = Sunday
  if ( ucDow )                          // not Sunday
      ucDy = 8 - ucDow;
  if ( ucDay > ucDy ) return 0;
  if ( ucDay < ucDy ) return 1;
  // Check the time for 0200.
  if ( ucHour >= 2 ) return 0;
  return 1;
}

/*--------------------------------------------------------------------------
  isleapyear(...)

  Leap Year
  The rules for a leap year are:

    1. every four years is a leap year, except
    2. every century is not a leap year, except
    3. every fourth century is a leap year.

  Code that converts dates from a Julian style format to an external format
  and vice versa must incorporate the above three rules in order to deal with
  leap years correctly.

  Typically you should find #4, #100 and #400 in the conversion algorithm.
  Alternatively you may find \4, \100 and \400 or even /4, /100 and /400.

  If your code is missing the #400 expression then it may not cater for the
  leap year in the year 2000.

  If your code only has a #4 expression then it may not cater for the absence
  of a leap year in 1800, 1900 and 2100. Such code is good for dates between
  1 March 1900 and 31 December 2099. This is acceptable for many
  applications, however it is unlikely to be found in cases where a date
  before 1900 is required, such as for entry and validation of a person's
  date of birth.

  Input: iYear = the year (yy), we make assumption about century

  Returns: 0 = no, 1 = yes
--------------------------------------------------------------------------*/
static uint8_t isleapyear( uint8_t ucYear )
{
  uint8_t	uiRemainder, uiYear, uiResult;
  
  if ( ucYear > 60 )
    uiYear = 1900 + (unsigned int) ucYear;
  else  
    uiYear = 2000 + (unsigned int) ucYear;

  uiRemainder = uiYear % 100;
  if ( !uiRemainder )
  {
    // This is a century.

    uiResult = uiYear / 100;          // which century
    uiRemainder = uiResult % 4;       // is it a 4th century?
                                      // not / is a 4th century
    return ( uiRemainder ) ? 0 : 1;
  }

  uiRemainder = uiYear % 4;

  return ( uiRemainder ) ? 0 : 1;    // not / is a 4th year
}

/*---------------------------------------------------------------
  week_of_year(...)

  Calculates the week of the year given the month and day.
  Actually this is not the right way to do this, since I
  disregard the day of the week the year starts on, so many 
  outputs will be low by 1, and week 51 may have extra outputs.
  For this application it doesn't matter. Feel free to correct
  the code if you like.

  Input: Year  - 0..99  
		 Month - 1..12  
         Day   - 1..31

  Returns: 0..51
---------------------------------------------------------------*/
uint8_t week_of_year( uint8_t ucYear, uint8_t ucMonth, uint8_t ucDay )
{
  uint16_t	uiDays=0;
  uint8_t	ii, rc;

  if ( ucYear > 99 ) return 0;
  if ( ucMonth < 1 || ucMonth > 12 ) return 0;
  if ( ucDay < 1 || ucDay > 31 ) return 0;

  // Add up the days in each month
  if ( ucMonth > 1 )
  {
    for ( ii=1; ii<ucMonth; ++ii )
	  uiDays += (uint16_t) days_in_month( ii, ucYear );
  }

  uiDays += ucDay;
  --uiDays;         // make days relative to 0

  rc = (uint8_t) ( uiDays / 7 );
  if ( rc > 51 ) rc = 51;

  return rc;
}

/*---------------------------------------------------------------
  dayofweek(...)

  Calculates the day of the week given the year, month, day.
  Based on the method documented in Wikipedia:
  http://en.wikipedia.org/wiki/Calculating_the_day_of_the_week
  We make one simplification: The century is always 2000-2099,
  so C = 6.

  Input: Year  - 0..99
         Month - 1..12
         Day   - 1..31

  Returns: 0..6 where 0 = Sunday, 6 = Saturday
---------------------------------------------------------------*/
static uint8_t dayofweek( uint8_t ucYear, uint8_t ucMonth, uint8_t ucDay )
{
  uint16_t CC, YY, MM, DD, RR;
  uint8_t ii, ly;

  // Look up 2000s in the centuries table
  CC = 6;

  // Divide last 2 digits of yea by 4 and drop fractional part.
  YY = ucYear / 4;

  // Look up the month in the months table
  ly = isleapyear( ucYear );
  ii = ucMonth - 1;
  if ( ly )
    MM = months_table_ly[ii];
  else
    MM = months_table[ii];

  // Add all numbers from steps 1-4 to the day of the month

  DD = CC + (unsigned int) ucYear + YY + MM + (unsigned int) ucDay;

  // Divide the sum from step 5 by 7 and find the remainder:

  RR = DD % 7;

  return (unsigned char) RR;
}
