/*---------------------------------------------------------------------------
  encoder.c
 
  Released under the MIT License  - see
  http://www.opensource.org/licenses/mit-license.php
 
  Handles the I/O to the rotary encoder. Invoked every 1 ms.

  20210526: moved encoder I/O code here
 ---------------------------------------------------------------------------*/
#include <avr/io.h>
#include "main.h"

// Global variables

extern volatile uint8_t ucEventFlags;			// system event flags

static volatile uint8_t  ucDebouncedState;		    // debounced state of the switches
static volatile uint8_t  ucPreviousState=0; 		// previous switch state
static volatile uint8_t  ucNrUps=0, ucNrDowns=0; 	// Encoder logic
static volatile uint16_t uiMsPressed;			    // how long encoder button pressed

// internal functions
static void analyze_switch(void);

/*---------------------------------------------------------------------------
  encoder()

  We come here every 4 ms to process the rotary encoder.
---------------------------------------------------------------------------*/
void encoder()
{
  static volatile uint8_t ucFirst=1;
  static volatile uint8_t ucIndex=0;             	// index into ucState
  static volatile uint8_t ucState[MAX_CHECKS] =  	// array containing bounce status
  { 0, 0, 0, 0, 0 };

  uint8_t     ucResult, ucSwIn=0;
  uint8_t     ii;

  // We use the technique documented in "A Guide to Debouncing" by
  // Jack Ganssle to debounce up to 8 inputs (3 in this case).

  // Read current switch states
  if ( !test_bit( ENCODER_PORT, ENCODER_A ) )
    set_bit( ucSwIn, ENC_SWITCH_A );
  if ( !test_bit( ENCODER_PORT, ENCODER_B ) )
    set_bit( ucSwIn, ENC_SWITCH_B );
  if ( !test_bit( ENC_SW_PORT, ENC_SW_PB ) )
    set_bit( ucSwIn, ENC_SWITCH_PB );

  ucState[ucIndex] = ucSwIn;

  ++ucIndex;
  if ( ucIndex == MAX_CHECKS )
    ucIndex = 0;

  // And all columns of the array
  for ( ii=0, ucResult=0xff; ii<(MAX_CHECKS-1); ++ii )
    ucResult = ucResult & ucState[ii];

  ucDebouncedState = ucResult;

  // The 1st time only, set the previous switch state to the
  // current switch state.
  if ( ucFirst )
  {
    ucPreviousState = ucDebouncedState;
    ucFirst = 0;
  }

  // Since we're here every millisecond, keep count
  // of how many milliseconds the encoder button is pressed
  if ( test_bit( ucDebouncedState, ENC_SWITCH_PB ) )
    ++uiMsPressed;
  else
    uiMsPressed = 0;

  if ( ucDebouncedState != ucPreviousState )
    analyze_switch();		// convert switch change to event
}

/*-----------------------------------------------------------------
  analyze_switch()

  Analyzes the rotary encoder to determine if the shaft rotation
  is cw or ccw, or if the pushbutton has been pressed.

  Note. Like the CTS Series 290 Encoder, the EC11-1B-18T encoder
  outputs four states for each rotary detent instead of one as the
  Greyhill encoder does. So in effect you get four "cws" or four 
  "ccws" for each detent. To compensate, we count to 4.
  
  Sorry for my pedantic coding style. I have been shown a much more
  elegant way of doing this, but it suits my sequential way of
  thinking.
-----------------------------------------------------------------*/
static void analyze_switch()
{
  uint8_t	ucNow, ucThen;

  if ( test_bit( ucDebouncedState, ENC_SWITCH_PB ) && 
       !test_bit( ucPreviousState, ENC_SWITCH_PB ) )
  {
    // The encoder button has been pressed
	set_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
  }

  // Convert the position of the encoder to a number 0..3
  ucNow = ucDebouncedState & 0x03;
  ucThen = ucPreviousState & 0x03;

  // Given the A output in bit 0 and B output in bit 1 positions, the
  // encoder generates the values 0, 1, 3, 2 as it is rotated clockwise.

  if ( ucNow == 0 )
  {
    if ( ucThen == 2 )
	{
	  ++ucNrUps;
	  ucNrDowns = 0;
	}
	else
	  if ( ucThen == 1 )
	  {
	    ++ucNrDowns;
		ucNrUps = 0;
	  }
  }
  else
	if ( ucNow == 1 )
	{
      if ( ucThen == 0 )
	  {
	    ++ucNrUps;
		ucNrDowns = 0;
	  }
	  else
	    if ( ucThen == 3 )
		{
		  ++ucNrDowns;
		  ucNrUps = 0;
		}
	}
	else
	  if ( ucNow == 3 )
	  {
    	if ( ucThen == 1 )
		{
	      ++ucNrUps;
		  ucNrDowns = 0;
		}
		else
	 	  if ( ucThen == 2 )
		  {
		    ++ucNrDowns;
		    ucNrUps = 0;
		  }
	  }
	  else	// ucNow == 2
	  {
    	if ( ucThen == 3 )
		{
	      ++ucNrUps;
		  ucNrDowns = 0;
		}
		else
	 	  if ( ucThen == 0 )
		  {
			++ucNrDowns;
			ucNrUps = 0;
		  }
	  }

  if ( ucNrUps == 4 )
  {
	set_bit( ucEventFlags, EVENT_ENCODER_DOWN );
	ucNrUps = ucNrDowns = 0;
  }

  if ( ucNrDowns == 4 )
  {
	set_bit( ucEventFlags, EVENT_ENCODER_UP );
	ucNrUps = ucNrDowns = 0;
  }

  ucPreviousState = ucDebouncedState;
}
