/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  Copyright (c) Jack Botner and licensed under the MIT license; see
  http://opensource.org/licenses/mit-license.php

  Nokia_Timer by Jack Botner

  program.c

  Routines to run the programming dialogs.

  20211215 Move encoder processing to main
-----------------------------------------------------------------*/
#include <avr/io.h>
#define F_CPU 8000000UL
#include <util/delay_basic.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include "main.h"
#include "n3310_lcd.h"
#include "nju6355.h"

// Global variables

extern MY_EEPROM_DATA stEepromVars;		// variables kept in EEPROM
extern uint8_t        ucEventFlags;
extern RTCPARMS       stRtc;
extern uint8_t        ucTempOverrideOn;		// override switch

// Internal functions

static void programTimer1(void);
static void setTimeDate( void );
static uint8_t update_number( const char *pcText, uint8_t, uint8_t, uint8_t );
static void programTimer2(void);
static uint8_t timerOnOff( const char *pcText, uint8_t initial_value );

/*-----------------------------------------------------------------
  main_menu()

  Displays the main menu. Processes the selection.
-----------------------------------------------------------------*/
void main_menu()
{
  char			cCont, cMode, cMenuIndex=0;
  char      cIndexLimit=4;

  clear_bit( ucEventFlags, EVENT_ENCODER_UP );
  clear_bit( ucEventFlags, EVENT_ENCODER_DOWN );
  clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
  
  while (1)
  {
	  LCD_clear();
	  cCont = 1;
  	cMode = ( !cMenuIndex ) ? MENU_HIGHLIGHT : MENU_NORMAL;
	  LCD_write_string( 1, 0, "Return", cMode );
	  cMode = ( cMenuIndex == 1 ) ? MENU_HIGHLIGHT : MENU_NORMAL;
	  LCD_write_string( 1, 1, "Set Clock", cMode );
	  cMode = ( cMenuIndex == 2 ) ? MENU_HIGHLIGHT : MENU_NORMAL;
  	LCD_write_string( 1, 2, "Set Timer 1", cMode );
  	cMode = ( cMenuIndex == 3 ) ? MENU_HIGHLIGHT : MENU_NORMAL;
  	LCD_write_string( 1, 3, "Set Timer 2", cMode );
	  cMode = ( cMenuIndex == 4 ) ? MENU_HIGHLIGHT : MENU_NORMAL;
	  LCD_write_string( 1, 4, "Clock Test", cMode );
#ifdef DIAGNOSTICS
	  cMode = ( cMenuIndex == 5 ) ? MENU_HIGHLIGHT : MENU_NORMAL;
	  LCD_write_string( 1, 5, "Diagnostics", cMode );
    ++cIndexLimit;
#endif

	  while ( cCont )
	  {
        if ( test_bit( ucEventFlags, EVENT_DEBOUNCE_IO ) )
        {
          encoder();      // check for encoder I/O
          clear_bit( ucEventFlags, EVENT_DEBOUNCE_IO );
        }

	    if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
	    {
		    LCD_clear();
		    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
		    cCont = 0;
		    switch ( cMenuIndex )
		    {
		      case 0:			// return
		        return;			// exit now
		      case 1:			// set clock
		        setTimeDate();
		        return;			// exit now
		      case 2:			// set timer 1
		        programTimer1();
			      return;	  
		      case 3:			// set timer 2
		        programTimer2();
		        return;
		      case 4:			// clock test
		        rtc_test();		// this function does not return.
            break;
#ifdef DIAGNOSTICS
          case 5:     // debug
            diagnostics();
            return;
#endif
		    }    // end switch
	    }      // end if

	    if ( test_bit( ucEventFlags, EVENT_ENCODER_DOWN ) )
	    {
		    if ( cMenuIndex == cIndexLimit )
		      cMenuIndex = 0;
	    	else
		    ++cMenuIndex;

		    clear_bit( ucEventFlags, EVENT_ENCODER_DOWN );
		    break;
	    }

	    if ( test_bit( ucEventFlags, EVENT_ENCODER_UP ) )
	    {
	  	  if ( !cMenuIndex )
		      cMenuIndex = cIndexLimit;
		    else
		      --cMenuIndex;

		    clear_bit( ucEventFlags, EVENT_ENCODER_UP );
		    break;
	    }
	  }      // end while
  }        // end while
}

/*-----------------------------------------------------------------
  setTimeDate()
  
  Note that time must be set in 24 hour format, and seconds
  will always be set to 0.
-----------------------------------------------------------------*/
static void setTimeDate()
{
  char		cUpdate=0;
  uint8_t   ucNr;

  // Set the year. Only last 2 digits supported.
  ucNr = update_number( "Set Year", stRtc.rtc_year, 0, 99 );
  if ( stRtc.rtc_year != ucNr )
  {
	cUpdate = 1;
    stRtc.rtc_year = ucNr;
  }

  // Set the month.
  ucNr = update_number( "Set Month", stRtc.rtc_month, 1, 12 );
  if ( stRtc.rtc_month != ucNr )
  {
	cUpdate = 1;
	stRtc.rtc_month = ucNr;
  }

  // Set the day.
  ucNr = update_number( "Set Day", stRtc.rtc_day, 1, 31 );
  if ( stRtc.rtc_day != ucNr )
  {
	cUpdate = 1;
	stRtc.rtc_day = ucNr;
  }

  // Set the day of the week.
  ucNr = update_number( "Day of Week", stRtc.rtc_dayweek, 1, 7 );
  if ( stRtc.rtc_dayweek != ucNr )
  {
	cUpdate = 1;
	stRtc.rtc_dayweek = ucNr;
  }

  // Set the hours. Must be done in 24 hour format.
  ucNr = update_number( "Set Hour 0-23", stRtc.rtc_hours, 0, 23 );
  if ( stRtc.rtc_hours != ucNr )
  {
	cUpdate = 1;
    stRtc.rtc_hours = ucNr;
  }

  // Set the minutes.
  ucNr = update_number( "Set Minutes", stRtc.rtc_mins, 0, 59 );
  if ( stRtc.rtc_mins != ucNr )
  {
	cUpdate = 1;
    stRtc.rtc_mins = ucNr;
  }

  // Set the seconds, always set to 0.
  ucNr = 0;
  if ( stRtc.rtc_secs != ucNr )
  {
	cUpdate = 1;
	stRtc.rtc_secs = ucNr;
  }

  // Update the rtc.
  if ( cUpdate )
    rtc_write();
	
  LCD_clear();
}

/*-----------------------------------------------------------------
  programTimer1()

  Timer 1 operates on a pre-set table of on/off times. We
  just need to turn it on or off. 
-----------------------------------------------------------------*/
static void programTimer1()
{
  uint8_t	ucNr;
  
  ucNr = timerOnOff( "Timer1", stEepromVars.ucTimer1Enabled );
  if ( ucNr != stEepromVars.ucTimer1Enabled )  
  {
	  stEepromVars.ucTimer1Enabled = ucNr;
	  if ( ucNr )
      set_bit( TIMER_LED_PORT, TIMER1_PORT_OUT );
    else
	    clear_bit( TIMER_LED_PORT, TIMER1_PORT_OUT );

    set_bit( ucEventFlags, EVENT_UPDATE_EEPROM );
    ucTempOverrideOn = 0;     // cancel any override
  }
}

/*-----------------------------------------------------------------
  programTimer2()

  Timer2 has an on and off time, and day of week option.
-----------------------------------------------------------------*/
static void programTimer2()
{
  uint8_t	ucNr, cUpdate=0;

  ucNr = timerOnOff( "Timer2", stEepromVars.ucTimer2Enabled );
  if ( ucNr != stEepromVars.ucTimer2Enabled )
  {
	  cUpdate = 1;
	  stEepromVars.ucTimer2Enabled = ucNr;
  }

  if ( stEepromVars.ucTimer2Enabled )
  {
    // Set the on time hours
    ucNr = update_number( "On time HH", stEepromVars.ucOnTimeHH, 0, 23 );
    if ( stEepromVars.ucOnTimeHH != ucNr )
    {
	    cUpdate = 1;
	    stEepromVars.ucOnTimeHH = ucNr;
    }
  	
    // Set the on time minutes
    ucNr = update_number( "On time MM", stEepromVars.ucOnTimeMM, 0, 59 );
    if ( stEepromVars.ucOnTimeMM != ucNr )
    {
	    cUpdate = 1;
	    stEepromVars.ucOnTimeMM = ucNr;
    }
  
    // Set the off time hours
    ucNr = update_number( "Off time HH", stEepromVars.ucOffTimeHH, 0, 23 );
    if ( stEepromVars.ucOffTimeHH != ucNr )
    {
	    cUpdate = 1;
	    stEepromVars.ucOffTimeHH = ucNr;
    }
  
    // Set the off time minutes
    ucNr = update_number( "Off time MM", stEepromVars.ucOffTimeMM, 0, 59 );
    if ( stEepromVars.ucOffTimeMM != ucNr )
    {
	    cUpdate = 1;
	    stEepromVars.ucOffTimeMM = ucNr;
    }

    // Set the day of week (0=every day)
    ucNr = update_number( "Day of Week", stEepromVars.ucDayofWeek, 0, 7 );
    if ( stEepromVars.ucDayofWeek != ucNr )
    {
      cUpdate = 1;
      stEepromVars.ucDayofWeek = ucNr;
    }
  }

  // Update the eeprom.
  if ( cUpdate )
  {
	  if ( stEepromVars.ucTimer2Enabled )
      set_bit( TIMER_LED_PORT, TIMER2_PORT_OUT );
	  else
      clear_bit( TIMER_LED_PORT, TIMER2_PORT_OUT );

    set_bit( ucEventFlags, EVENT_UPDATE_EEPROM );
    ucTempOverrideOn = 0;     // cancel any override
  }
  
  LCD_clear();
}

/*-----------------------------------------------------------------
  update_number(...)

  Show a number after the prompt text. Allow the encoder to
  increase or decrease the number. Return the result.
-----------------------------------------------------------------*/
static uint8_t update_number( const char *pcText, uint8_t initial_value, 
							  uint8_t low_bound, uint8_t high_bound )
{
  uint8_t	ucResult=255, ucTemp=initial_value;
  char		cWork[12];

  while ( 1 )
  {
	  if ( ucResult != ucTemp )
	  {
	  // Display current value
      LCD_clear();
      LCD_write_string( 1, 1, (char *) pcText, MENU_NORMAL );
      utoa( (uint16_t) ucTemp, cWork, 10 );
      LCD_write_string_big( 1, 3, (char *) cWork, MENU_HIGHLIGHT );  
	    ucResult = ucTemp;
	  }

    if ( test_bit( ucEventFlags, EVENT_DEBOUNCE_IO ) )
    {
        encoder();      // check for encoder I/O
        clear_bit( ucEventFlags, EVENT_DEBOUNCE_IO );
    }

  	if ( test_bit( ucEventFlags, EVENT_ENCODER_DOWN ) )
	  {
	    clear_bit( ucEventFlags, EVENT_ENCODER_DOWN );
	    // increment the value
	    ++ucTemp;
	    if ( ucTemp > high_bound )
	    ucTemp = low_bound;
	    continue;
	  }

  	if ( test_bit( ucEventFlags, EVENT_ENCODER_UP ) )
	  {
	    clear_bit( ucEventFlags, EVENT_ENCODER_UP );
	    // decrement the value
	    if ( ucTemp == low_bound )
		  ucTemp = high_bound;
	    else
		  --ucTemp;
	    continue;
	  }

	  if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
	  {
	    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
	    break;
	  }
  }

  return ucResult;
}

/*-----------------------------------------------------------------
  timerOnOff()

  Manages the prompt to set a timer on or off.
  
  initial_value: 0 = off, 1 = on
  return value:  0 = off, 1 = on
-----------------------------------------------------------------*/
static uint8_t timerOnOff( const char *pcText, uint8_t initial_value )
{
  char			cMode, cMenuIndex, cString[12];
  uint8_t		ucResult=2, ucTemp=initial_value;

  while (1)
  {
	  if ( ucResult != ucTemp )
	  {
      LCD_clear();
      cMenuIndex = ( ucTemp ) ? 0 : 1;
      cMode = ( !cMenuIndex ) ? MENU_HIGHLIGHT : MENU_NORMAL;
	    strcpy( cString, pcText );
	    strcat( cString, " ON" );
      LCD_write_string( 1, 1, cString, cMode );
      cMode = ( cMenuIndex == 1 ) ? MENU_HIGHLIGHT : MENU_NORMAL;
	    strcpy( cString, pcText );
	    strcat( cString, " OFF" );
      LCD_write_string( 1, 2, cString, cMode );
	    ucResult = ucTemp;
	  }

      if ( test_bit( ucEventFlags, EVENT_DEBOUNCE_IO ) )
      {
        encoder();      // check for encoder I/O
        clear_bit( ucEventFlags, EVENT_DEBOUNCE_IO );
      }

      if ( test_bit( ucEventFlags, EVENT_ENCODER_DOWN ) )
	  {
	    if ( ucTemp )
	     ucTemp = 0;
	    else
		  ++ucTemp;

	    clear_bit( ucEventFlags, EVENT_ENCODER_DOWN );
	    continue;
    }

	  if ( test_bit( ucEventFlags, EVENT_ENCODER_UP ) )
	  {
	    if ( ucTemp )
		    ucTemp = 0;
	    else
		    ucTemp = 1;

	    clear_bit( ucEventFlags, EVENT_ENCODER_UP );
	    continue;
	  }

    if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
    {
	    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
	    break;
    }
  }
  
  return ucResult;
}

 